@@ -7,7 +7,8 @@ import math
 import re
 
 from . import _msgs as msgs
-from ._helpers import null_terminate, SimpleError
+from ._helpers import null_terminate, SimpleError, SimpleString
+from ._zset import ZSet
 
 MAX_STRING_SIZE = 512 * 1024 * 1024
 SUPPORTED_COMMANDS = dict()  # Dictionary of supported commands name => Signature
@@ -303,6 +304,31 @@ class StringTest:
             raise SimpleError(msgs.INVALID_MIN_MAX_STR_MSG)
 
 
+class StreamRangeTest:
+    """Argument converter for sorted set LEX endpoints."""
+
+    def __init__(self, value, exclusive):
+        self.value = value
+        self.exclusive = exclusive
+
+    @staticmethod
+    def parse_id(id_str: str):
+        if isinstance(id_str, bytes):
+            id_str = id_str.decode()
+        timestamp, sequence = (int(x) for x in id_str.split('-'))
+        return timestamp, sequence
+
+    @classmethod
+    def decode(cls, value):
+        if value == b'-':
+            return cls(BeforeAny(), True)
+        elif value == b'+':
+            return cls(AfterAny(), True)
+        elif value[:1] == b'(':
+            return cls(cls.parse_id(value[1:]), True)
+        return cls(cls.parse_id(value), False)
+
+
 class Signature:
     def __init__(self, name, func_name, fixed, repeat=(), flags=""):
         self.name = name
@@ -414,3 +440,20 @@ def fix_range_string(start, end, length):
         end = max(0, end + length)
     end = min(end, length - 1)
     return start, end + 1
+
+
+def key_value_type(key):
+    if key.value is None:
+        return SimpleString(b'none')
+    elif isinstance(key.value, bytes):
+        return SimpleString(b'string')
+    elif isinstance(key.value, list):
+        return SimpleString(b'list')
+    elif isinstance(key.value, set):
+        return SimpleString(b'set')
+    elif isinstance(key.value, ZSet):
+        return SimpleString(b'zset')
+    elif isinstance(key.value, dict):
+        return SimpleString(b'hash')
+    else:
+        assert False  # pragma: nocover
