@@ -2,6 +2,9 @@ try:
     from jsonpath_ng.ext import parse  # noqa: F401
     from redis.commands.json.path import Path  # noqa: F401
     from ._json_mixin import JSONCommandsMixin, JSONObject  # noqa: F401
-except ImportError:
+except ImportError as e:
+    if e.name != 'jsonpath_ng':
+        raise e
+
     class JSONCommandsMixin:
         pass
